open Core.Std
open Ast

(* GADT experimentation *)

type _ typ =
    | GBool : bool -> bool typ
    | GInt  : int -> typ
    | GChar : char -> typ
    | GFloat : float -> typ

type _ texpr =
    | GTyp : 'a typ -> 'a typ
    | GIf : bool texpr * 'a texpr * 'a texpr -> 'a texpr
    | GEq : 'a texpr * 'a texpr -> bool texpr
    | GLt : 'a texpr * 'a texpr -> bool texpr
    | GGt : 'a texpr * 'a texpr -> bool texpr
    | GEq : 'a texpr * 'a texpr -> bool texpr
    | GNeq : 'a texpr * 'a texpr -> bool texpr
    | GSub : 'a texpr * 'a texpr -> texpr
    | GMul : 'a texpr * 'a texpr -> texpr
    | GDiv : 'a texpr * 'a texpr -> texpr
    | GAdd : 'a texpr * 'a texpr -> texpr
    | GMod : 'a texpr * 'a texpr -> texpr

let rec inter : type a. a texpr -> a = function 
	| GTyp (GBool b) -> b
	| GTyp (GInt i) -> i
	| GIf (b, l, r) -> if inter b then inter l else inter r
	| GEq (a, b) -> (inter a) = (inter b)
	| GLt (a,b) -> a < b 
	| GGt (a,b) -> a > b 
    | GEq (a, b) -> a = b 
    | GSub (a, b) -> a -b 
    | GAdd (a, b) -> a +b 
    | GMul (a, b) -> a *b 
    | GDiv (a, b) -> a /b 
    | GMod (a, b) -> a %b 
    |  _ -> failwith (sprintf "[ERROR]: something wrong " (Ast.string_of_op op) (Ast.string_of_type t1))
