(* 
 * Description:
 * This is the entry point / top level of Scala-lite 
 * handeling user input arguemnt option inlucindg the folowing
 * compilation
 * interpreter
 * otuput LLVM-IR
 * otput ast 
 *
 * Author; DL, <dl2997@columbia.edu>
 *
 * Last modified: 20160811
 *
 * Dependencies:
 *     llvm3.8.1
 *     OCaml4.0.3.0
 *     Ocaml llvm binding module
 *
 * Usage: 
 *        $ cd ../src
 *        $ make 2>&1 | tee make$(date +s%).log 
 *        $ ./testall.sh 
 *
 * NOTE: this compiler experiment originally was largely built based upon 
 * microC by SE from Columbia University; hence the Licesnc of this compiler would
 * adopt similar Licence of microC, if there is any.
 *)
open Llvm
open Llvm_target
open Llvm_scalar_opts

type action = Ast | LLVM_IR | Scalalc | Scalali

let _ =
  let action = if Array.length Sys.argv > 1 then
    List.assoc Sys.argv.(1) [ ("-a", Ast);	(* Print the AST only *)
			      ("-l", LLVM_IR);  (* Generate LLVM, don't check *)
                  ("-i", Scalali);
			      ("-c", Scalalc) ] (* Generate, check LLVM IR *)
  else Scalalc in
  let lexbuf = Lexing.from_channel stdin in
  let ast = Parser.program Scanner.token lexbuf in
  Semant.check ast;
  match action with
    Ast -> print_string (Ast.string_of_program ast)
  | LLVM_IR -> print_string (Llvm.string_of_llmodule (Codegen.translate ast))
  (*| Scalali -> Interpreter.main_loop (Codegen.translate ast) *)
  | Scalalc -> let m = Codegen.translate ast in
    Llvm_analysis.assert_valid_module m;
    print_string (Llvm.string_of_llmodule m) 
  | _ -> ()
